function d=score_local_graph(true_parent_file, found_parent_file, is_probe_net)
%d=score_local_graph(true_parent_file, found_parent_file, is_probe_net)
% Inputs:
% true_parent_file --   graph of the true network. 
% found_parent_file	--  the local graph in the same format.
% is_probe_net --       flag indicating whether this is real data with
% probe (is_probe_net=1) or arficial data (is_probe_net=0). For real data
% with probes, the parents of the target (real variables) are excluded from
% the score calculation.
% Returns 
% d --                  the average edit distance for the local neighborhood
% variables.
%
% File formats:
% The target is numbered 0 and the other nodes are numbered with 
% the corresponding feature number. The node number is followed by a colon
% and then comes the list of parents.
% Example (for LUCAS):
% 0: 1 5 
% 1: 3 4 
% 2: 1  
% 6: 5  
% 8: 6 9 
% 9: 0 11  
% 11: 0 10 

% Isabelle Guyon -- isabelle@clopinet.com -- October 2008

if nargin<3, is_probe_net=0; end
true_var=[];

% Load the lists of parents
% parents{k} contains the parents of node (k-1); nodes 0 is the target;
% other nodes are numbered in the order of the features
true_parents=read_parents(true_parent_file);
if is_probe_net
    true_var=true_parents{1};
end
found_parents=read_parents(found_parent_file, length(true_parents));

% Convert into an Alexander's style graph 
% true_graph is a matrix such that G(i,j) =1 iff i->j.
% All the node numbers are shifted by 1, i.e. 1 is now the target and the
% nodes are feat_num+1.
true_graph=my_graph(true_parents);
found_graph=my_graph(found_parents);

% Compute the shortest path strings encoding the relationship to the target
% The string is a series of u and d, u meaning we are going up and d down
% in ancestry/descendance. Hence 'u' is a parent, 'd' is a child, 'uu' a
% grand-parent, 'du' a spouse, etc. Note that there may be multiple paths
% of the same length, in which case we keep all of them.
true_relationships=target_relationship(true_graph); 
found_relationships=target_relationship(found_graph); 

% The score uses an edit distance between the strings.
% Let us assume that for a given node/ feature
% we have t true shortest path p1, p2, ... pk ... pt
% and f found shortest path q1, q2, ... qh ... qf.
% For each true path, we get a partial score sk=4-dk
% dk being min_h(EditDist(pk, qh). The overall score
% for the given feature/node will be s = min(1, (t/f)) * max_k sk.
% Then we average over all nodes and take 4-result
% to get the average distance between correct and found
% shortest path for the nodes in the local neighborhood.
% Example: p1=ud p2=du q1=du q2=uu
%          s1=3 s2=4 s=4
% Example: p1=ud p2=du q1=du 
%          s1=2 s2=4 s=4
% Example: p1=ud p2=du q1=dud q2=ddd q3=duu 
%          s1=2 s2=3 s=2
% Example: p1=ud q1=du q2=ud
%          s1=4 s=2
% For the nodes outsides the local neighborhood, the string is 'xxxx'
% Example: p1=xxxx q1=du q2=uu
% s1=0 s=0
% Example: p1=xxxx q1=xxxx
% s1=4 s=4
% Example: p1=ud q1=xxxx
% s1=0 s=0

debug=1;

s=0;
for k=2:length(true_relationships) %1 is the target
    if ismember(k-1, true_var), 
        if debug, fprintf('%d: **TRUE VAR**\n', k-1); end
        continue; % skip the real variables
    end 
    P=true_relationships{k}; t=length(P);
    Q=found_relationships{k}; f=length(Q);
    D=[];
    for i=1:t
        for j=1:f
            D(i,j)=EditDist(P{i}, Q{j});
        end
    end
    d=min(min(D));
    sloc=min(1, t/f)*(4-d);
    if debug
        fprintf('%d: ', k-1);
        for i=1:t, fprintf('p%d=%s ', i, P{i}); end
        for j=1:f, fprintf('q%d=%s ', j, Q{j}); end
        fprintf('d=%d t=%d f=%d s=%g\n', d, t, f, sloc);
    end
    s=s+sloc;
end
nf=length(true_relationships)-1;
nt=length(true_var);

d = 4-s/(nf-nt);
